﻿using System;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;


namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// Sub-flow : Print sales delivery documents
    /// </summary>
    public class WorkflowScript_PrintSalesDeliveryDocumentsScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> ListOfDeliveries;
        public ReadOnlyBinder<string> LocationCode;
        public ReadWriteBinder<int?> PickListEntry;
        public ReadWriteBinder<int?> RouteEntry;




      
        private CultureInfo current_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
        }

        private PmxOseCompany currentPmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
        }

        private ISboProviderService m_sboProviderService;



        public WorkflowScript_PrintSalesDeliveryDocumentsScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            ProdumexError pmxError = null;
            Message msg = null;
            int nrOfDocsPrinted = 0;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                 this.DefaultCultureInfo, BuildParamCollection(
                 "MessageKey", "MSG_GETTING_REPORTS_TO_PRINT"));


            nrOfDocsPrinted = 0;

            string locationCode = null;
            if (LocationCode != null)
            {
                locationCode = LocationCode.Get();
            }

            int? routeKey = null;

            if (RouteEntry != null)
            {
                routeKey = RouteEntry.Get();
            }

            bool askReprint = false;

        Step_PrintDeliveryDocuments:

            #region PRINT DELIVERY DOCUMENTS

            if (ListOfDeliveries.Get() != null &&
               ListOfDeliveries.Get().Length > 0)
            {
              

                foreach (DocumentRef docRef in ListOfDeliveries.Get())
                {
                    if (docRef.DocType == PmxObjectTypes.PmxSalesShipping ||
                        docRef.DocType == ((int)Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.SboObjectType.SalesDeliveryNote).ToString())
                    {
                        int docEntry = docRef.DocEntry;

                        PmxPrintReportEventType printEvent = PmxPrintReportEventType.ShippingSalesDeliveryNoteCreated;

                        if (docRef.DocType == PmxObjectTypes.PmxSalesShipping)
                        {
                            printEvent = PmxPrintReportEventType.ShippingPmxSalesShippingCreated;
                        }

                        //execute print event
                        nrOfDocsPrinted += this.ExecuteMethodWithDbConnection<int>(out pmxError, true, false,
                          delegate(PmxDbConnection conn, object[] parameters)
                          {
                              PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                              return printProv.PrintReportsForEvent(printEvent,
                                  locationCode, session.DeviceInstanceId, docEntry, true, out askReprint);
                          });
                        if (pmxError != null)
                        {
                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                this.DefaultCultureInfo, BuildParamCollection(
                                "PmxError", pmxError,
                                "ShowButton", true));
                            msg = WaitForMessage();

                        }
                    }
                }


            }

            goto Step_PrintPickListDocument;

            #endregion


        Step_PrintPickListDocument:

            #region PRINT PICKLIST DOCUMENT


            if (PickListEntry.Get().HasValue &&
               PickListEntry.Get().Value > 0)
            {
                //execute print event
                nrOfDocsPrinted += this.ExecuteMethodWithDbConnection<int>(out pmxError, true, false,
                  delegate(PmxDbConnection conn, object[] parameters)
                  {
                      PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                      return printProv.PrintReportsForEvent(PmxPrintReportEventType.ShippingPicklistShipped,
                          locationCode, session.DeviceInstanceId, PickListEntry.Get().Value, out askReprint);
                  });
                if (pmxError != null)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        this.DefaultCultureInfo, BuildParamCollection(
                        "PmxError", pmxError,
                        "ShowButton", true));
                    msg = WaitForMessage();

                }

                //try to get closed route

                routeKey = this.ExecuteMethodWithDbConnection<int?>(out pmxError, true, false,
                  delegate(PmxDbConnection conn, object[] parameters)
                  {
                      PmxRouteProvider routeProv = new PmxRouteProvider(conn);
                      return routeProv.GetClosedRouteForPickList(PickListEntry.Get().Value);
                  });

           }

            goto Step_PrintRouteDocument;

            #endregion

        Step_PrintRouteDocument:

            #region PRINT ROUTE DOCUMENT

            if (routeKey.HasValue &&
               routeKey.Value > 0)
            {
                //execute print event
                nrOfDocsPrinted += this.ExecuteMethodWithDbConnection<int>(out pmxError, true, false,
                  delegate(PmxDbConnection conn, object[] parameters)
                  {
                      PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                      return printProv.PrintReportsForEvent(PmxPrintReportEventType.ShippingRouteShipped,
                          locationCode, session.DeviceInstanceId, routeKey.Value , out askReprint);
                  });
                if (pmxError != null)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        this.DefaultCultureInfo, BuildParamCollection(
                        "PmxError", pmxError,
                        "ShowButton", true));
                    msg = WaitForMessage();

                }


            }

            #endregion


            if (nrOfDocsPrinted > 0 && askReprint)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    this.DefaultCultureInfo, BuildParamCollection(
                    "TitleKey", "MSG_TITLE_REPRINT_DOCUMENT",
                    "MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT"
                    ));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_PrintDeliveryDocuments;
                }
            }

        }

        #endregion

        private T ExecuteMethodWithDbConnection<T>(out ProdumexError pmxError, bool useSboConnection, bool useUserNameOfConnectionString, DelegateWithPmxDbConnection<T> methodDelegate, params object[] parameters)
        {
            T result = default(T);
            pmxError = null;

            try
            {
                result = m_sboProviderService.InvokeMethodWithDbConnection<T>(useSboConnection, useUserNameOfConnectionString, null, null,
                    methodDelegate, parameters);
            }
            catch (TargetInvocationException ex)
            {
                MethodInfo methodInfo = methodDelegate.Method;
                if (ex.InnerException is ProdumexException)
                {
                    s_log.Warn("Execute -- The invoked method threw a produmex exception: ", ex.InnerException);
                    pmxError = ((ProdumexException)ex.InnerException).Error;
                }
                else if (ex.InnerException == null)
                {
                    s_log.Warn("Execute -- The invoked method threw an exception: ", ex);
                    pmxError = Produmex.Foundation.Wwf.Sbo.ProdumexErrors.ExceptionInvokingSboMethod(s_log, methodInfo, ex.Message);
                }
                else
                {
                    s_log.Warn("Execute -- The invoked method threw a TargetInvocationException with inner exception: ", ex.InnerException);
                    pmxError = Produmex.Foundation.Wwf.Sbo.ProdumexErrors.ExceptionInvokingSboMethod(s_log, methodInfo, ex.InnerException.Message);
                }
            }
            catch (TransactionAbortedException ex)
            {
                MethodInfo methodInfo = methodDelegate.Method;
                pmxError = Produmex.Foundation.Wwf.Sbo.ProdumexErrors.ExceptionInvokingSboMethod(s_log, methodInfo, ex.InnerException == null ? ex.Message : ex.InnerException.Message);
            }
            catch (ProdumexException ex)
            {
                if (ex.Error != null)
                {
                    pmxError = ex.Error;
                }
                else
                {
                    MethodInfo methodInfo = methodDelegate.Method;
                    pmxError = Produmex.Foundation.Wwf.Sbo.ProdumexErrors.ExceptionInvokingSboMethod(s_log, methodInfo, ex.Message);
                }
            }
            catch (Exception ex)
            {
                MethodInfo methodInfo = methodDelegate.Method;
                pmxError = Produmex.Foundation.Wwf.Sbo.ProdumexErrors.ExceptionInvokingSboMethod(s_log, methodInfo, ex.Message);
            }

            return result;
        }

       
    }
}
