﻿//css_ref Produmex.Foundation.SlimScreen.Interfaces;
//css_ref Produmex.Foundation.Wwf.Sbo;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;



using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.MLS.BusinessObjects.Definitions.Tables;

namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger( MethodInfo.GetCurrentMethod().DeclaringType );

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>


        private static class Definitions
        {
            //report definitions
            public const string REPORT_PATH_LABEL = @"MLSColliLabel.rpt";


            /// <summary>
            /// 
            /// </summary>
            public const int NUMBER_OF_LABELS_PALLET = 2;

            /// <summary>
            /// 
            /// </summary>
            public const int NUMBER_OF_LABELS_HALF_PALLET = 2;

            /// <summary>
            /// 
            /// </summary>
            public const int NUMBER_OF_LABELS_COLLI = 1;


            /// <summary>
            /// 
            /// </summary>
            public const string PALLET = "PALLET";


            /// <summary>
            /// 
            /// </summary>
            public const string HALF_PALLET = "1/2 PAL";



            /// <summary>
            /// 
            /// </summary>
            public const string COLLI = "COLLIE";




        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base( parent, factory )
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter( "Session" ) as Session;
            ISboProviderService sboProviderService = GetScopeParameter( "<WwfService>ISboService" ) as ISboProviderService;

            string initialErrorKey = null;

        Step_EnterQuantityByType:

            Collection<PackagingTypeInfo> listOfPackagingTypeInfos = new Collection<PackagingTypeInfo>();

            PackagingTypeInfo infoCollie = new PackagingTypeInfo();
            infoCollie.InitialQuantity = 0;
            infoCollie.NumberOfDecimals = 0;
            infoCollie.PackagingTypeCode = Definitions.COLLI;
            infoCollie.PackagingTypeName = Definitions.COLLI;
            infoCollie.QuantityPerPack = 1;
            infoCollie.ShowOnScreen = true;

            listOfPackagingTypeInfos.Add(infoCollie);

            PackagingTypeInfo infoHalfPallet = new PackagingTypeInfo();
            infoHalfPallet.InitialQuantity = 0;
            infoHalfPallet.NumberOfDecimals = 0;
            infoHalfPallet.PackagingTypeCode = Definitions.HALF_PALLET;
            infoHalfPallet.PackagingTypeName = Definitions.HALF_PALLET;
            infoHalfPallet.QuantityPerPack = 1;
            infoHalfPallet.ShowOnScreen = true;

            listOfPackagingTypeInfos.Add(infoHalfPallet);

            PackagingTypeInfo infoPallet = new PackagingTypeInfo();
            infoPallet.InitialQuantity = 0;
            infoPallet.NumberOfDecimals = 0;
            infoPallet.PackagingTypeCode = Definitions.PALLET;
            infoPallet.PackagingTypeName = Definitions.PALLET;
            infoPallet.QuantityPerPack = 1;
            infoPallet.ShowOnScreen = true;

            listOfPackagingTypeInfos.Add(infoPallet);


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "Vul het aantal logistieke eenheden in",
                    "ListOfPackagingTypeInfos", listOfPackagingTypeInfos,
                    "IsForEnteringDefaultPackagingQuantity", false,
                    "MaximumQuantity", -1
                    ));
             Message msg = WaitForMessage();

            initialErrorKey = null;
            if (!msg.Name.EndsWith(".QuantitiesEntered"))
            {
                initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
                goto Step_EnterQuantityByType;
            }

            PackagingTypeInfos listOfPackagingTypeInfosFromScreen = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");


        Step_PrintLabel:

            #region PRINT LABEL


            string reportPath = Definitions.REPORT_PATH_LABEL;


            int totalNumber = 0;


            foreach (PackagingTypeInfo info in listOfPackagingTypeInfosFromScreen.PackagingTypeInfoList.Values)
            {
                totalNumber += Convert.ToInt32(info.Quantity);
            }


            sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
               delegate(PmxDbConnection conn, object[] parameters)
               {
                   PmxSalesDeliveryNoteProvider prov = new PmxSalesDeliveryNoteProvider((SboConnection)conn);

                   using (PmxSalesDeliveryNote delNote = prov.GetBO(DocumentRefs.Get()[0].DocEntry))
                   {
                       foreach (PackagingTypeInfo info in listOfPackagingTypeInfosFromScreen.PackagingTypeInfoList.Values)
                       {
                           switch (info.PackagingTypeCode)
                           {
                               case Definitions.PALLET:
                                   delNote[PmxDocumentColumnsMLS.NumberOfPallets.NAME] = Convert.ToInt32(info.Quantity);
                                   break;
                               case Definitions.HALF_PALLET:
                                   delNote[PmxDocumentColumnsMLS.NumberOfHalfPallets.NAME] = Convert.ToInt32(info.Quantity);
                                   break;
                               case Definitions.COLLI:
                                   delNote[PmxDocumentColumnsMLS.NumberOfColli.NAME] = Convert.ToInt32(info.Quantity);
                                   break;
                               default:
                                   break;

                           }
                       }

                       prov.UpdateBO(delNote);
                   }

                   return null;
               });


            int currentNumber = 1;
            foreach (PackagingTypeInfo info in listOfPackagingTypeInfosFromScreen.PackagingTypeInfoList.Values)
            {
                int subNumberOfLabels = Convert.ToInt32(info.Quantity);
                //print label
                for (int iLoop = 1; iLoop <= subNumberOfLabels; iLoop++)
                {
                    try
                    {
                        s_log.InfoFormat("Print rapport {0} voor colli {1}", reportPath, string.Format("{0}/{1} {2}", currentNumber, totalNumber, info.PackagingTypeCode));
                        //print label
                        sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null,
                           delegate(PmxDbConnection conn, object[] parameters)
                           {
                               PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

                               Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
                               reportParameters.Add(new ReportParameter("@docEntry", DocumentRefs.Get()[0].DocEntry));
                               reportParameters.Add(new ReportParameter("@colliType", info.PackagingTypeCode));
                               reportParameters.Add(new ReportParameter("@totalLabels", totalNumber));
                               reportParameters.Add(new ReportParameter("@currentLabel", currentNumber));
                               reportParameters.Add(new ReportParameter("@currentLabelColli", iLoop));
                               reportParameters.Add(new ReportParameter("@totalLabelsColli", subNumberOfLabels));


                               int numberOfLabels = 1;

                               switch (info.PackagingTypeCode)
                               {
                                   case Definitions.PALLET:
                                       numberOfLabels = Definitions.NUMBER_OF_LABELS_PALLET;
                                       break;
                                   case Definitions.HALF_PALLET:
                                       numberOfLabels = Definitions.NUMBER_OF_LABELS_HALF_PALLET;
                                       break;
                                   case Definitions.COLLI:
                                       numberOfLabels = Definitions.NUMBER_OF_LABELS_COLLI;
                                       break;
                                   default:
                                       break;
                               }

                               return prov.PrintReport(reportParameters, reportPath, numberOfLabels, session.DeviceInstanceId, null, false);
                               
                           });

                    

                        //Added sleep to avoid unprinted labels
                        Thread.Sleep(500);

                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        throw ex;
                    }

                    currentNumber++;
                }
            }



            

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".Yes"))
            {
                goto Step_PrintLabel;
            }


            #endregion


        }

        #endregion

 
        private class BuildQuery
        {
  
        }
    }
}
